#include "main_win.h"

MainWin::MainWin(const std::string& title) :
  mapwin("Map Setup", mapstorage, gd_canvas),
  newmapwin("Creating new map", gd_canvas), gd_canvas(mapstorage, mapwin)
{

  set_title (title);
  set_default_size(WIN_WIDTH,WIN_HEIGHT);
  add(vbox);

  ag=Gtk::ActionGroup::create();

  ag->add( Gtk::Action::create("FileMenu", "_File") );
  ag->add( Gtk::Action::create("FileNew", Gtk::Stock::NEW),
      sigc::mem_fun(*this, &MainWin::newFile));
  ag->add( Gtk::Action::create("FileOpen", Gtk::Stock::OPEN),
      sigc::mem_fun(*this, &MainWin::openFile));
  ag->add( Gtk::Action::create("FileSave", Gtk::Stock::SAVE),
      sigc::mem_fun(*this, &MainWin::saveFile));
  ag->add( Gtk::Action::create("FileSaveAs", Gtk::Stock::SAVE_AS),
      sigc::mem_fun(*this, &MainWin::saveFileAs));
  ag->add( Gtk::Action::create("Close", Gtk::Stock::CLOSE),
      sigc::mem_fun(*this, &MainWin::close));
  ag->add( Gtk::Action::create("Quit", Gtk::Stock::QUIT),
      sigc::mem_fun(*this, &MainWin::hide));

  ag->add( Gtk::Action::create("ViewMenu", "_View") );
  ag->add( Gtk::Action::create("ViewZoomIn", Gtk::Stock::ZOOM_IN),
      sigc::mem_fun(this->gd_canvas, &GraphDisplayerCanvas::zoomIn));
  ag->add( Gtk::Action::create("ViewZoomOut", Gtk::Stock::ZOOM_OUT),
      sigc::mem_fun(this->gd_canvas, &GraphDisplayerCanvas::zoomOut));
  ag->add( Gtk::Action::create("ViewZoomFit", Gtk::Stock::ZOOM_FIT),
      sigc::mem_fun(this->gd_canvas, &GraphDisplayerCanvas::zoomFit));
  ag->add( Gtk::Action::create("ViewZoom100", Gtk::Stock::ZOOM_100),
      sigc::mem_fun(this->gd_canvas, &GraphDisplayerCanvas::zoom100));
  
  ag->add( Gtk::Action::create("ShowMenu", "_Show") );
  ag->add( Gtk::Action::create("ShowMaps", "_Maps"),
      sigc::mem_fun(this->mapwin, &MapWin::show));

  Gtk::RadioAction::Group tool_group;
  ag->add( Gtk::RadioAction::create(tool_group, "MoveItem", Gtk::Stock::CONVERT, "Move"),
      sigc::bind( sigc::mem_fun ( this->gd_canvas, &GraphDisplayerCanvas::changeEditorialTool ), 0) );
  ag->add( Gtk::RadioAction::create(tool_group, "CreateNode", Gtk::Stock::NO, "Create node"),
      sigc::bind( sigc::mem_fun ( this->gd_canvas, &GraphDisplayerCanvas::changeEditorialTool ), 1) );
  ag->add( Gtk::RadioAction::create(tool_group, "CreateEdge", Gtk::Stock::REMOVE, "Create edge"),
      sigc::bind( sigc::mem_fun ( this->gd_canvas, &GraphDisplayerCanvas::changeEditorialTool ), 2) );
  ag->add( Gtk::RadioAction::create(tool_group, "EraseItem", Gtk::Stock::DELETE, "Delete"),
      sigc::bind( sigc::mem_fun ( this->gd_canvas, &GraphDisplayerCanvas::changeEditorialTool ), 3) );
  ag->add( Gtk::Action::create("EditEdgeMap", Gtk::Stock::PROPERTIES),
      sigc::bind( sigc::mem_fun ( this->gd_canvas, &GraphDisplayerCanvas::changeEditorialTool ), 4) );
  ag->add( Gtk::Action::create("EditNodeMap", Gtk::Stock::PREFERENCES),
      sigc::bind( sigc::mem_fun ( this->gd_canvas, &GraphDisplayerCanvas::changeEditorialTool ), 5) );
  ag->add( Gtk::Action::create("AddMap", Gtk::Stock::NEW),
      sigc::mem_fun ( this->newmapwin, &NewMapWin::show ) );

  uim=Gtk::UIManager::create();
  uim->insert_action_group(ag);
  add_accel_group(uim->get_accel_group());

  try
  {

    Glib::ustring ui_info =
      "<ui>"
      "  <menubar name='MenuBar'>"
      "    <menu action='FileMenu'>"
      "      <menuitem action='FileNew'/>"
      "      <menuitem action='FileOpen'/>"
      "      <menuitem action='FileSave'/>"
      "      <menuitem action='FileSaveAs'/>"
      "      <menuitem action='Close'/>"
      "      <menuitem action='Quit'/>"
      "    </menu>"
      "    <menu action='ViewMenu'>"
      "      <menuitem action='ViewZoomIn' />"
      "      <menuitem action='ViewZoomOut' />"
      "      <menuitem action='ViewZoomFit' />"
      "      <menuitem action='ViewZoom100' />"
      "    </menu>"
      "    <menu action='ShowMenu'>"
      "      <menuitem action='ShowMaps'/>"
      "    </menu>"
      "  </menubar>"
      "  <toolbar name='ToolBar'>"
      "    <toolitem action='FileNew' />"
      "    <toolitem action='FileOpen' />"
      "    <toolitem action='FileSave' />"
      "    <toolitem action='Close' />"
      "    <separator />"
      "    <toolitem action='ViewZoomIn' />"
      "    <toolitem action='ViewZoomOut' />"
      "    <toolitem action='ViewZoomFit' />"
      "    <toolitem action='ViewZoom100' />"
      "    <separator />"
      "    <toolitem action='MoveItem' />"
      "    <toolitem action='CreateNode' />"
      "    <toolitem action='CreateEdge' />"
      "    <toolitem action='EraseItem' />"
      "    <toolitem action='EditEdgeMap' />"
      "    <toolitem action='EditNodeMap' />"
      "    <toolitem action='AddMap' />"
      "  </toolbar>"
      "</ui>";

    uim->add_ui_from_string(ui_info);

  }
  catch(const Glib::Error& ex)
  {
    std::cerr << "building menus failed: " <<  ex.what();
  }

  Gtk::Widget* menubar = uim->get_widget("/MenuBar");
  if (menubar){
    vbox.pack_start(*menubar, Gtk::PACK_SHRINK);
  }

  Gtk::Widget* toolbar = uim->get_widget("/ToolBar");
  if (toolbar)
  {
    static_cast<Gtk::Toolbar*>(toolbar)->set_toolbar_style(Gtk::TOOLBAR_ICONS);
    vbox.pack_start(*toolbar, Gtk::PACK_SHRINK);
  }

  Gtk::ScrolledWindow* pScrolledWindow = manage(new Gtk::ScrolledWindow());
  pScrolledWindow->add(gd_canvas);
  vbox.pack_start(*pScrolledWindow);

  tooltips.set_tip(*(uim->get_widget("/ToolBar/CreateNode")),"Create Node");
  tooltips.enable();

  show_all_children();
}

void MainWin::newFile()
{
  if (mapstorage.modified)
  {
    Gtk::MessageDialog mdialog("", false, Gtk::MESSAGE_WARNING,
        Gtk::BUTTONS_NONE);
    mdialog.set_message("<b>Save changes before closing?</b>", true);
    mdialog.add_button("Close _without Saving", Gtk::RESPONSE_REJECT);
    mdialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    mdialog.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_ACCEPT);
    switch (mdialog.run())
    {
      case Gtk::RESPONSE_CANCEL:
        return;
      case Gtk::RESPONSE_REJECT:
        break;
      case Gtk::RESPONSE_ACCEPT:
        saveFile();
        break;
    }
  }
  gd_canvas.clear();
  mapstorage.clear();
  mapwin.update();
}

void MainWin::openFile()
{
  if (mapstorage.modified)
  {
    Gtk::MessageDialog mdialog("", false, Gtk::MESSAGE_WARNING,
        Gtk::BUTTONS_NONE);
    mdialog.set_message("<b>Save changes before closing?</b>", true);
    mdialog.add_button("Close _without Saving", Gtk::RESPONSE_REJECT);
    mdialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    mdialog.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_ACCEPT);
    switch (mdialog.run())
    {
      case Gtk::RESPONSE_CANCEL:
        return;
      case Gtk::RESPONSE_REJECT:
        break;
      case Gtk::RESPONSE_ACCEPT:
        saveFile();
        break;
    }
  }
  gd_canvas.clear();
  mapstorage.clear();
  Gtk::FileChooserDialog fcdialog("Open File");
  fcdialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  fcdialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_ACCEPT);
  if (fcdialog.run() == Gtk::RESPONSE_ACCEPT)
  {
    Glib::ustring filename = fcdialog.get_filename();
    mapstorage.readFromFile(filename);
    mapstorage.file_name = filename;
    mapstorage.modified = false;
    gd_canvas.drawGraph();
    mapwin.update();
  }
}

void MainWin::saveFile()
{
  if (mapstorage.file_name == "") {
    saveFileAs();
  }
  else
  {
    mapstorage.writeToFile(mapstorage.file_name);
    mapstorage.modified = false;
  }
}

void MainWin::saveFileAs()
{
  Gtk::FileChooserDialog fcdialog("Save File", Gtk::FILE_CHOOSER_ACTION_SAVE);
  fcdialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  fcdialog.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_ACCEPT);
  if (fcdialog.run() == Gtk::RESPONSE_ACCEPT)
  {
    Glib::ustring filename = fcdialog.get_filename();
    mapstorage.file_name = filename;
    mapstorage.writeToFile(filename);
    mapstorage.modified = false;
  }
}

void MainWin::close()
{
  if (mapstorage.modified)
  {
    Gtk::MessageDialog mdialog("", false, Gtk::MESSAGE_WARNING,
        Gtk::BUTTONS_NONE);
    mdialog.set_message("<b>Save changes before closing?</b>", true);
    mdialog.add_button("Close _without Saving", Gtk::RESPONSE_REJECT);
    mdialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    mdialog.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_ACCEPT);
    switch (mdialog.run())
    {
      case Gtk::RESPONSE_CANCEL:
        return;
      case Gtk::RESPONSE_REJECT:
        break;
      case Gtk::RESPONSE_ACCEPT:
        saveFile();
        break;
    }
  }
  gd_canvas.clear();
  mapstorage.clear();
  mapwin.update();
}
